#include <stddef.h>
#include <signal.h>

struct kernel_sigaction {
	sighandler_t k_sa_handler;
	unsigned long sa_flags;
	void (*sa_restorer)(void);
	sigset_t sa_mask;
};

extern int rt_sigaction(int signum, const struct kernel_sigaction *kact,
		struct kernel_sigaction *oldact, size_t setsz);
extern void rt_sigreturn(void);

int sigaction(int signum, const struct sigaction *act, struct sigaction *oldact)
{
	struct kernel_sigaction kact, koldact;

	kact.k_sa_handler = act->sa_handler;
	kact.sa_flags = act->sa_flags | SA_RESTORER;
	kact.sa_restorer = &rt_sigreturn;
	kact.sa_mask = act->sa_mask;

	if (rt_sigaction(signum, &kact, &koldact, 8) < 0)
		return -1;

	if (oldact != NULL) {
		oldact->sa_handler = koldact.k_sa_handler;
		oldact->sa_flags = koldact.sa_flags;
		oldact->sa_mask = koldact.sa_mask;
		oldact->sa_restorer = NULL;
	}

	return 0;
}
