#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#define MAX_LINE 1024

/* use Rob Pike's simple regexp matcher */

int matchhere(const char *regexp, const char *text);
int matchstar(char c, const char *regexp, const char *text)
{
	do {
		if (matchhere(regexp, text))
			return 1;
	} while (*text != '\0' && (*text++ == c || c == '.'));
	return 0;
}

int matchhere(const char *regexp, const char *text)
{
	if (regexp[0] == '\0')
		return 1;
	if (regexp[1] == '*')
		return matchstar(regexp[0], regexp + 2, text);
	if (regexp[0] == '$' && regexp[1] == '\0')
		return text[0] == '\0';
	if (*text != '\0' && (regexp[0] == '.' || regexp[0] == *text))
		return matchhere(regexp + 1, text + 1);
	return 0;
}

int match(const char *regexp, const char *text)
{
	if (regexp[0] == '^')
		return matchhere(regexp + 1, text);
	do {
		if (matchhere(regexp, text))
			return 1;
	} while (*text++ != '\0');
	return 0;
}

int do_grep(const char *regexp, FILE *f, const char *name)
{
	char buf[MAX_LINE];
	int notfound = 1;
	char *p;

	while ( fgets(buf, MAX_LINE, f) ) {
		p = strchr(buf, '\n');
		if (p != NULL)
			*p = '\0';
		if (match(regexp, buf)) {
			notfound = 0;
			if (name)
				printf("%s:", name);
			printf("%s\n", buf);
		}
	}
	return notfound;
}


int main(int argc, char* argv[])
{
	int i;
	int printname = 0;
	FILE *f;
	int notfound = 1;

	if (argc < 2) {
		exit(1);
	}
	if (argc == 2) 
		return do_grep(argv[1], stdin, NULL);	
	printname = (argc > 3);
	for (i = 2; i < argc; i++) {
		if ( !(f = fopen(argv[i], "r")) ) {
			perror(argv[i]);
			continue;
		}
		notfound &= do_grep(argv[1], f, printname? argv[i] : NULL);
		fclose(f);
	}
	return notfound;
}
