#include <sys/types.h>
#include <dirent.h>
#include <inttypes.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/stat.h>
#include <stdlib.h>

/*
 * ls with options and directory name arguments
 *
 * We accept a single optional directory name. To simplify string processing
 * when doing stat() (for '-l'), we just chdir() to the directory.
 */

int main(int argc, char* argv[])
{
	DIR *dir;
	struct dirent* ent;
	int all = 0, inodes = 0, longl = 0;
	struct stat statbuf;
	char *dirname = ".";

	if (argc > 1)
		if (argv[1][0] == '-') {
			char* opt = &argv[1][1];
			while (*opt) {
				switch (*opt) {
				case 'a':
					all = 1;
					break;
				case 'i':
					inodes = 1;
					break;
				case 'l':
					longl = 1;
					break;
				default:
					fprintf(stderr, "Unknown option: %c\n", *opt);
					break;
				}
				opt++;
			}
			argc--;
			argv++;
		}

	if (argc > 1)
		dirname = argv[1];

	if ( (dir = opendir(dirname)) == NULL) {
		perror(dirname);
		exit(1);
	}

	if (longl && chdir(dirname) < 0) {
		perror(dirname);
		exit(1);
	}

	while ( (ent = readdir(dir)) ) {

		if (ent->d_name[0] == '.' && !all)
			continue;

		if (inodes)
			printf("%10lu ", ent->d_ino);

		if (longl) {
			if (stat(ent->d_name, &statbuf) < 0) {
				perror(ent->d_name);
			} else {
				printf("%10jo %3ju %10ju %10ju %5jd %jd ",
					(uintmax_t)statbuf.st_mode,
					(uintmax_t)statbuf.st_nlink,
					(uintmax_t)statbuf.st_uid,
					(uintmax_t)statbuf.st_gid,
					(intmax_t)statbuf.st_size,
					(intmax_t)statbuf.st_mtime);
			}
		}
		printf("%s\n", ent->d_name);
	}

	return 0;
}
