#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/select.h>
#include <fcntl.h>
#include <ctype.h>
#include <dirent.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <crypt.h>
#include <pwd.h>
#include <grp.h>

#define BUFSIZE 1024
#define NAMESZ 16

int pwless(const char *rhosts, struct sockaddr_in *remote)
{
	FILE *f;
	char buf[BUFSIZE];
	char r_ip[BUFSIZE];
	unsigned long r_addr;
	char *p;
	int rv;

	r_addr = remote->sin_addr.s_addr;
	snprintf(r_ip, BUFSIZE, "%ld.%ld.%ld.%ld",
			(r_addr      ) & 0xff,
			(r_addr >>  8) & 0xff,
			(r_addr >> 16) & 0xff,
			(r_addr >> 24) & 0xff);

	if (! (f = fopen(rhosts, "r")) )
		return 0;

	rv = 0;
	while (fgets(buf, BUFSIZE, f)) {
		if ( (p = strchr(buf, '\n')) )
			*p = '\0';
		if (!strcmp(buf, "+") || !strcmp(buf, r_ip)) {
			rv = 1;
			break;
		}
	}
	fclose(f);
	return rv;
}

void login(struct sockaddr_in *remote)
{
	char name[NAMESZ];
	char buf[BUFSIZE];
	char *p;
	char *pass;
	struct passwd *pw;
	char salt[3];
	struct group *gr;
	gid_t ttygr;

	printf("login: ");
	fgets(name, NAMESZ, stdin);
	if ( (p = strchr(name, '\n')) )
		*p = '\0';

	if ( (pw = getpwnam(name)) == NULL )
		exit(1);

	snprintf(buf, BUFSIZE, "%s/.rhosts", pw->pw_dir);
	if (!pwless(buf, remote)) {
		pass = getpass("Password: ");

		if (strcmp(pw->pw_passwd, "") != 0) {
			salt[0] = pw->pw_passwd[0];
			salt[1] = pw->pw_passwd[1];
			salt[2] = '\0';

			if (strcmp(crypt(pass, salt), pw->pw_passwd) != 0)
				exit (1);
		}
	}

	if ( (gr = getgrnam("tty")) != NULL )
		ttygr = gr->gr_gid;
	else
		ttygr = 0;

	setenv("USER", pw->pw_name, 1);

	fchown(0, pw->pw_uid, ttygr);
	setgid(pw->pw_gid);
	setuid(pw->pw_uid);
	fchmod(0, 0600);

	setenv("HOME", pw->pw_dir, 1);
	chdir(pw->pw_dir);

	setenv("SHELL", pw->pw_shell, 1);
	execl(pw->pw_shell, "-sh", NULL);
	exit(1);
}

void child(int con, struct sockaddr_in *remote)
{
	int fdm, max, rdok;
	char *ptr;
	fd_set readset;
	int n, m;
	char buf[BUFSIZE];

	if ( (fdm = open("/dev/ptmx", O_RDWR)) < 0) {
		perror("/dev/ptmx");
		exit(1);
	}

	if (grantpt(fdm) < 0) {
		perror("grantpt");
		exit(1);
	}

	if (unlockpt(fdm) < 0) {
		perror("unlockpt");
		exit(1);
	}

	if ( (ptr = ptsname(fdm)) == NULL) {
		perror("ptsname");
		exit(1);
	}

	switch (fork()) {
	case -1:
		perror("fork()");
		return;
	case 0:
		if (setsid() < 0)
			perror("setsid");
		close(0);
		if (open(ptr, O_RDONLY) < 0) {
			perror(ptr);
			exit(1);
		}

		close(1);
		if (open(ptr, O_WRONLY) < 0) {
			perror(ptr);
			exit(1);
		}
		close(2);
		if (open(ptr, O_WRONLY) < 0) {
			perror(ptr);
			exit(1);
		}

		signal(SIGCHLD, SIG_DFL);

		login(remote);
		break;
	default:
		break;
	}

	max = con;
	if (fdm > max)
		max = fdm;
	for (;;)  {
		FD_ZERO(&readset);
		FD_SET(con, &readset);
		FD_SET(fdm, &readset);

		if (select(max + 1, &readset, NULL, NULL, NULL) < 0) {
			perror("select");
			exit(1);
		}

		if (FD_ISSET(con, &readset)) {
			n = read(con, buf, BUFSIZE);
			if (n <= 0)
				break;
			while (n > 0) {
				m = write(fdm, buf, n);
				if (m < 0) {
					m = n;
				}
				n -= m;
			}
			rdok = 1;
		}

		if (FD_ISSET(fdm, &readset)) {
			n = read(fdm, buf, BUFSIZE);
			if (n <= 0)
				break;
			while (n > 0) {
				m = write(con, buf, n);
				if (m < 0) {
					m = n;
				}
				n -= m;
			}
		}
	}
	unlink(ptr);
	close(con);
	close(fdm);
	wait(0);
	exit(0);
}

int main()
{
	int lstn;
	int enable;
	struct sockaddr_in lstn_addr;

	switch (fork()) {
	case 0:
		setsid();
		break;
	case -1:
		perror("fork()");
		exit(1);
	default:
		exit(0);
	}

	lstn = socket(AF_INET, SOCK_STREAM, 0);
	if (lstn < 0) {
		perror("socket");
		return 1;
	}

	enable = 1;
	if (setsockopt(lstn, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof(enable)) < 0) {
		perror("setsockopt");
		return 1;
	}

	memset(&lstn_addr, 0, sizeof(lstn_addr));

	lstn_addr.sin_family = AF_INET;
	lstn_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	lstn_addr.sin_port = htons(1514);

	if (bind(lstn, (struct sockaddr *)&lstn_addr, sizeof(lstn_addr)) < 0) {
		perror("bind");
		return 1;
	}

	if (listen(lstn, 10) < 0) {
		perror("listen");
		return 1;
	}

	signal(SIGCHLD, SIG_IGN);

	for (;;) {
		struct sockaddr_in remote;
		socklen_t remote_sz = sizeof(remote);

		int con = accept(lstn, (struct sockaddr *)&remote, &remote_sz);
		if (con < 0) {
			perror("accept");
			return 1;
		}

		switch (fork()) {
		case -1:
			perror("fork");
			return 1;
		case 0:
			child(con, &remote);
			break;
		default:
			close(con);
			break;
		}
	}
	return 0;

error:
	fprintf(stderr, "dup() failed\n");
	exit(1);
}
