#include <sys/wait.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h> /* for perror() */
#include <unistd.h>

#include "input.h"

/* shell that splits the line into arguments */

#define MAX_ARGS 10

void space(struct input *s)
{
	while (s->ibuf == ' ' || s->ibuf == '\t')
		inext(s, 0);
}

int word_delim(int c)
{
	return     c == EOF || isspace(c)
		;
}

int word(struct input *s)
{
	newtoken(s);
	while (!word_delim(s->ibuf)) {
		switch (s->ibuf) {
		default:
			oput(s, s->ibuf);
			inext(s, 0);
			break;
		}
	}
	// terminate the word
	oput(s, '\0');
	return 1;
}

int token(struct input *s)
{
	space(s);
	switch (s->ibuf) {
	case EOF:
	case '\n':
		return '\n';
	default:
		return word(s);
	}
}

int main()
{
	struct input s;
	int t, c_argc;
	char *c_argv[MAX_ARGS + 1];
	char *tok;

	iinit(&s, 0);
	while (nextcmd(&s) > 0) {
		c_argc = 0;
		while ( (t = token(&s)) != '\n') {
			if (s.error)
				break;
			switch (t) {
			case 1:
				if (c_argc >= MAX_ARGS) {
					fprintf(stderr, "too many arguments\n");
					s.error = 1;
					break;
				}
				tok = lasttoken(&s);
				c_argv[c_argc++] = tok;
			}
		}
		if (s.error) {
			if (!s.interactive)
				exit(1);
			while (token(&s) != '\n')
					;
			continue;
		}
		c_argv[c_argc] = NULL;

		if (fork()) {
			wait(0);
		} else {
			if (c_argc < 1)
				exit(0);
			execvp(c_argv[0], c_argv);
			perror(c_argv[0]);
			exit(1);
		}
	}
	iclose(&s);
	return 0;
}
