#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <stdlib.h>

#define BUFSIZE 512

extern char* ptsname(int fd);

int main(int argc, char* argv[])
{
	int fdm;
	char *ptr;
	fd_set readset;
	int n, m;
	char buf[BUFSIZE];
	struct termios stermios;

	if (argc < 2) {
		fprintf(stderr, "missing argument\n");
		exit(1);
	}

	if (tcgetattr(0, &stermios) < 0) {
		perror("tcgetattr");
		exit(1);
	}

	stermios.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
	stermios.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
	stermios.c_cflag &= ~(CSIZE | PARENB);
	stermios.c_cflag &= CS8;
	stermios.c_oflag &= ~(OPOST);
	stermios.c_cc[VMIN] = 1;
	stermios.c_cc[VTIME] = 0;

	if (tcsetattr(0, TCSAFLUSH, &stermios) < 0) {
		perror("tcsettattr");
		exit(1);
	}

	if ( (fdm = open("root/dev/ptmx", O_RDWR)) < 0) {
		perror("root/dev/ptmx");
		exit(1);
	}

	if (grantpt(fdm) < 0) {
		perror("grantpt");
		exit(1);
	}

	if (unlockpt(fdm) < 0) {
		perror("unlockpt");
		exit(1);
	}

	if ( (ptr = ptsname(fdm)) == NULL) {
		perror("ptsname");
		exit(1);
	}

	if (symlink(ptr, argv[1]) < 0) {
		perror("symlink");
		exit(1);
	}


	for (;;) {
		FD_ZERO(&readset);
		FD_SET(0, &readset);
		FD_SET(fdm, &readset);

		if (select(fdm + 1, &readset, NULL, NULL, NULL) < 0) {
			perror("select");
			exit(1);
		}

		if (FD_ISSET(0, &readset)) {
			n = read(0, buf, BUFSIZE);
			while (n > 0) {
				m = write(fdm, buf, n);
				if (m < 0) {
					m = n;
				}
				n -= m;
			}
		}

		if (FD_ISSET(fdm, &readset)) {
			n = read(fdm, buf, BUFSIZE);
			while (n > 0) {
				m = write(1, buf, n);
				if (m < 0) {
					m = n;
				}
				n -= m;
			}
		}
	}
}
