#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <stdlib.h>
#include <string.h>

#define BUFSIZE 512

extern char* ptsname(int fd);

int main(int argc, char* argv[])
{
	int fdm;
	fd_set readset;
	int n, m;
	char buf[BUFSIZE];
	struct termios stermios, ttermios;
	struct sockaddr_in con_addr;
	const char *dst = "127.0.0.1";

	if (argc == 2) {
	       dst = argv[1];	
	} else if (argc > 2) {
		fprintf(stderr, "too many arguments\n");
		exit(1);
	}

	fdm = socket(AF_INET, SOCK_STREAM, 0);
	if (fdm < 0) {
		perror("socket");
		return 1;
	}

	memset(&con_addr, 0, sizeof(con_addr));

	con_addr.sin_family = AF_INET;
	inet_pton(AF_INET, dst, &con_addr.sin_addr.s_addr);
	con_addr.sin_port = htons(1514);

	if (connect(fdm, (struct sockaddr *)&con_addr, sizeof(con_addr)) < 0) {
		perror("connect");
		return 1;
	}

	if (tcgetattr(0, &stermios) < 0) {
		perror("tcgetattr");
		exit(1);
	}

	memcpy(&ttermios, &stermios, sizeof(stermios));
	ttermios.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);
	ttermios.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);
	ttermios.c_cflag &= ~(CSIZE | PARENB);
	ttermios.c_cflag &= CS8;
	ttermios.c_oflag &= ~(OPOST);
	ttermios.c_cc[VMIN] = 1;
	ttermios.c_cc[VTIME] = 0;

	if (tcsetattr(0, TCSAFLUSH, &ttermios) < 0) {
		perror("tcsettattr");
		exit(1);
	}

	for (;;) {
		FD_ZERO(&readset);
		FD_SET(0, &readset);
		FD_SET(fdm, &readset);

		if (select(fdm + 1, &readset, NULL, NULL, NULL) < 0) {
			perror("select");
			break;
		}

		if (FD_ISSET(0, &readset)) {
			n = read(0, buf, BUFSIZE);
			if (n <= 0)
				break;
			while (n > 0) {
				m = write(fdm, buf, n);
				if (m < 0) {
					m = n;
				}
				n -= m;
			}
		}

		if (FD_ISSET(fdm, &readset)) {
			n = read(fdm, buf, BUFSIZE);
			if (n <= 0)
				break;
			while (n > 0) {
				m = write(1, buf, n);
				if (m < 0) {
					m = n;
				}
				n -= m;
			}
		}
	}

	if (tcsetattr(0, TCSAFLUSH, &stermios) < 0) {
		perror("tcsettattr");
		exit(1);
	}
	return 0;
}
